;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SMW Directional Coins (sprite 45), by imamelia
;;
;; This is a disassembly of sprite 45 in SMW, the directional coins.
;;
;; Uses first extra bit: NO
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

XSpeed:
db $00,$00,$F0,$10

YSpeed:
db $F0,$10,$00,$00

ControllerStateData1:
db $00,$03,$02,$00,$01,$03,$02,$00,$00,$03,$02,$00,$00,$00,$00,$00

ControllerStateData2:
db $01,$00,$03,$02

!BlockTile = $2E
!CoinTile = $E8

CoinTiles2:
db $EA,$FA,$EA

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
JSR DirectionalCoinsMain
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DirectionalCoinsMain:

LDA $64			;
PHA			; preserve the current sprite priority setting
LDA $1540,x		;
CMP #$30			; if the timer is greater than 30...
BCC NoChangePriority	;
LDA #$10			;
STA $64			; change the sprite priority
NoChangePriority:		;

PEI ($1C)			; preserve the screen boundary settings
LDA $1C			;
CLC			;
ADC #$01		; temporarily shift the screen boundary down 1 pixel...
STA $1C			; ...why?
LDA $1D			;
ADC #$00		;
STA $1D			;

LDA $14AD		; if the blue P-switch timer is set...
BNE DrawBlock		; make the sprite look like a block
JSR CoinGFX		;
BRA PullStuff		;

DrawBlock:		;

JSR BlockGFX		;

PullStuff:			;

PLA			;
STA $1C			;
PLA			;
STA $1D			;
PLA			;
STA $64			;

LDA $9D			; if sprites are locked...
BNE CheckObjContact	; skip the next part of code

LDA $13			;
AND #$03		; every 4 frames...
BNE NoErase		;
DEC $190C		; decrement the directional coin timer
BNE NoErase		; if it is zero...

TerminateCoins:		;

STZ $190C		;
STZ $14C8,x		; erase the sprite

LDA $14AD		;
ORA $14AE		; and if neither P-switch timer is set...
BNE NoResetMusic		;
LDA $0DDA		;
BMI NoResetMusic		; and the music isn't "abnormal"...
STA $1DFB		; change the music back to normal
NoResetMusic:		;
RTS			;

NoErase:			;

LDY $C2,x		;
LDA XSpeed,y		;
STA $B6,x			; set the sprite X speed
LDA YSpeed,y		;
STA $AA,x		; and Y speed

JSL $01801A		; update sprite Y position without gravity
JSL $018022		; update sprite X position without gravity

LDA $15			; controller data
AND #$0F		; use the right, left, down, and up buttons to determine where the sprite should move
BEQ NoSetState		;
TAY			;
LDA ControllerStateData1,y	;
TAY			;
LDA ControllerStateData2,y	;
CMP $C2,x		;
BEQ NoSetState		;
TYA			;
STA $151C,x		;

NoSetState:		;

LDA $D8,x		;
AND #$0F		; if the sprite is centered on a tile...
STA $00			;
LDA $E4,x		;
AND #$0F		; both horizontally and vertically...
ORA $00			;
BNE CheckObjContact	;

LDA $151C,x		;
STA $C2,x		;

LDA $E4,x		;
STA $9A			; generate a coin tile
LDA $14E0,x		;
STA $9B			;
LDA $D8,x		;
STA $98			;
LDA $14D4,x		;
STA $99			;
LDA #$06			;
STA $9C			;
JSL $00BEB0		;
RTS			;

CheckObjContact:	;

JSL $019138	; check contact with objects

LDA $B6,x	; if the sprite is moving horizontally...
BNE XContact	; check its contact horizontally

LDA $18D7	; if the sprite is touching a Map16tile not on page 0...
BNE EndCoins	; erase it
LDA $185F	;
CMP #$25		; if it is not touching tile 25...
BNE EndCoins	; erase it
RTS		;

XContact:		;

LDA $1862	; if the sprite is touching a Map16tile not on page 0...
BNE EndCoins	; erase it
LDA $1860	;
CMP #$25		; if it is not touching tile 25...
BEQ Return1	; erase it

EndCoins:

JSR TerminateCoins

Return1:
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; graphics routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

CoinGFX:

JSR GetDrawInfo

LDA $00		;
STA $0300,y	;
LDA $01		;
STA $0301,y	;
LDA #!CoinTile	;
STA $0302,y	;
LDA $15F6,x	;
ORA $64		;
STA $0303,y	;

TXA		;
CLC		;
ADC $14		;
LSR #2		;
AND #$03	;
BNE SmallTiles	;

LDY #$02		;
LDA #$00		;
JSL $81B7B3	;
RTS		;

SmallTiles:	;

PHX		;
TAX		;
LDA $00		;
CLC		;
ADC #$04	;
STA $0300,y	;
STA $0304,y	;

LDA $01		;
CLC		;
ADC #$08	;
STA $0305,y	;

LDA CoinTiles2,x	;
STA $0302,y	;
STA $0306,y	;

LDA $0303,y	;
ORA #$80		;
STA $0307,y	;

PLX		;
LDY #$00		;
LDA #$01		;
JSL $81B7B3	;
RTS		;

BlockGFX:

JSR GetDrawInfo

LDA #!BlockTile	;
STA $0302,y	;

LDX $15E9	;
LDA $00		;
STA $0300,y	; no X displacement
LDA $01		;
STA $0301,y	; no Y displacement

LDA $15F6,x	;
ORA $64		;
STA $0303,y	;

TYA		;
LSR #2		;
TAY		;

LDA #$02		;
ORA $15A0,x	;
STA $0460,y	; set the tile size

PHK		;
PER $0006	;
PEA $8020	;
JML $81A3DF	; set up some stuff in OAM

RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; subroutines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Table1:              db $0C,$1C
Table2:              db $01,$02
Table3:              db $40,$B0
Table6:              db $01,$FF
Table4:              db $30,$C0,$A0,$C0,$A0,$F0,$60,$90,$30,$C0,$A0,$80,$A0,$40,$60,$B0
Table5:              db $01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$00,$01,$FF

SubOffscreenX0:
LDA #$00
;BRA SubOffscreenMain
;SubOffscreenX1:
;LDA #$02
;BRA SubOffscreenMain
;SubOffscreenX2:
;LDA #$04
;BRA SubOffscreenMain
;SubOffscreenX3:
;LDA #$06
;BRA SubOffscreenMain
;SubOffscreenX4:
;LDA #$08
;BRA SubOffscreenMain
;SubOffscreenX5:
;LDA #$0A
;BRA SubOffscreenMain
;SubOffscreenX6:
;LDA #$0C
;BRA SubOffscreenMain
;SubOffscreenX7:
;LDA #$0E

SubOffscreenMain:

STA $03

JSR SubIsOffscreen
BEQ Return2

LDA $5B
LSR
BCS VerticalLevel
LDA $D8,x
CLC
ADC #$50
LDA $14D4,x
ADC #$00
CMP #$02
BPL EraseSprite
LDA $167A,x
AND #$04
BNE Return2
LDA $13
AND #$01
ORA $03
STA $01
TAY
LDA $1A
CLC
ADC Table4,y
ROL $00
CMP $E4,x
PHP
LDA $1B
LSR $00
ADC Table5,y
PLP
SBC $14E0,x
STA $00
LSR $01
BCC Label20
EOR #$80
STA $00
Label20:
LDA $00
BPL Return2

EraseSprite:
LDA $14C8,x
CMP #$08
BCC KillSprite
LDY $161A,x
CPY #$FF
BEQ KillSprite
LDA #$00
STA $1938,y
KillSprite:
STZ $14C8,x
Return2:
RTS

VerticalLevel:

LDA $167A,x
AND #$04
BNE Return2
LDA $13
LSR
BCS Return2
AND #$01
STA $01
TAY
LDA $1C
CLC
ADC Table3,y
ROL $00
CMP $D8,x
PHP
LDA $1D
LSR $00
ADC Table6,y
PLP
SBC $14D4,x
STA $00
LDY $02
BEQ Label22
EOR #$80
STA $00
Label22:
LDA $00
BPL Return2
BMI EraseSprite

SubIsOffscreen:
LDA $15A0,x
ORA $186C,x
RTS

GetDrawInfo:

STZ $186C,x
STZ $15A0,x
LDA $E4,x
CMP $1A
LDA $14E0,x
SBC $1B
BEQ OnscreenX
INC $15A0,x
OnscreenX:
LDA $14E0,x
XBA
LDA $E4,x
REP #$20
SEC
SBC $1A
CLC
ADC.w #$0040
CMP #$0180
SEP #$20
ROL A
AND #$01
STA $15C4,x
BNE Invalid

LDY #$00
LDA $1662,x
AND #$20
BEQ OnscreenLoop
INY
OnscreenLoop:
LDA $D8,x
CLC
ADC Table1,y
PHP
CMP $1C
ROL $00
PLP
LDA $14D4,x
ADC #$00
LSR $00
SBC $1D
BEQ OnscreenY
LDA $186C,x
ORA Table2,y
STA $186C,x
OnscreenY:
DEY
BPL OnscreenLoop
LDY $15EA,x
LDA $E4,x
SEC
SBC $1A
STA $00
LDA $D8,x
SEC
SBC $1C
STA $01
RTS

Invalid:
PLA
PLA
RTS











